@@ -1,16 +1,16 @@
 package CPANPLUS::Dist::Build;
 
 use strict;
-use warnings;
 use vars    qw[@ISA $STATUS $VERSION];
 @ISA =      qw[CPANPLUS::Dist];
 
+use CPANPLUS::inc;
 use CPANPLUS::Internals::Constants;
 
 ### these constants were exported by CPANPLUS::Internals::Constants
 ### in previous versions.. they do the same though. If we want to have
 ### a normal 'use' here, up the dependency to CPANPLUS 0.056 or higher
-BEGIN {
+BEGIN { 
     require CPANPLUS::Dist::Build::Constants;
     CPANPLUS::Dist::Build::Constants->import()
         if not __PACKAGE__->can('BUILD') && __PACKAGE__->can('BUILD_DIR');
@@ -21,7 +21,6 @@ use CPANPLUS::Error;
 use Config;
 use FileHandle;
 use Cwd;
-use version;
 
 use IPC::Cmd                    qw[run];
 use Params::Check               qw[check];
@@ -30,13 +29,13 @@ use Locale::Maketext::Simple    Class => 'CPANPLUS', Style => 'gettext';
 
 local $Params::Check::VERBOSE = 1;
 
-$VERSION = '0.61_01';
+$VERSION = '0.01';
 
 =pod
 
 =head1 NAME
 
-CPANPLUS::Dist::Build - CPANPLUS plugin to install packages that use Build.PL
+CPANPLUS::Dist::Build
 
 =head1 SYNOPSIS
 
@@ -44,8 +43,8 @@ CPANPLUS::Dist::Build - CPANPLUS plugin to install packages that use Build.PL
                                 format  => 'CPANPLUS::Dist::Build',
                                 module  => $modobj,
                             );
-
-    $build->prepare;    # runs Build.PL
+                            
+    $build->prepare;    # runs Module::Build->new_from_context;                            
     $build->create;     # runs build && build test
     $build->install;    # runs build install
 
@@ -58,18 +57,18 @@ Using this package, you can create, install and uninstall perl
 modules. It inherits from C<CPANPLUS::Dist>.
 
 Normal users won't have to worry about the interface to this module,
-as it functions transparently as a plug-in to C<CPANPLUS> and will
+as it functions transparently as a plug-in to C<CPANPLUS> and will 
 just C<Do The Right Thing> when it's loaded.
 
 =head1 ACCESSORS
 
 =over 4
 
-=item C<parent()>
+=item parent()
 
 Returns the C<CPANPLUS::Module> object that parented this object.
 
-=item C<status()>
+=item status()
 
 Returns the C<Object::Accessor> object that keeps the status for
 this module.
@@ -83,35 +82,35 @@ All accessors can be accessed as follows:
 
 =over 4
 
-=item C<build_pl ()>
+=item build_pl ()
 
 Location of the Build file.
 Set to 0 explicitly if something went wrong.
 
-=item C<build ()>
+=item build ()
 
 BOOL indicating if the C<Build> command was successful.
 
-=item C<test ()>
+=item test ()
 
 BOOL indicating if the C<Build test> command was successful.
 
-=item C<prepared ()>
+=item prepared ()
 
-BOOL indicating if the C<prepare> call exited successfully
+BOOL indicating if the C<prepare> call exited succesfully
 This gets set after C<perl Build.PL>
 
-=item C<distdir ()>
+=item distdir ()
 
 Full path to the directory in which the C<prepare> call took place,
-set after a call to C<prepare>.
+set after a call to C<prepare>. 
 
-=item C<created ()>
+=item created ()
 
-BOOL indicating if the C<create> call exited successfully. This gets
+BOOL indicating if the C<create> call exited succesfully. This gets
 set after C<Build> and C<Build test>.
 
-=item C<installed ()>
+=item installed ()
 
 BOOL indicating if the module was installed. This gets set after
 C<Build install> exits successfully.
@@ -120,23 +119,28 @@ C<Build install> exits successfully.
 
 BOOL indicating if the module was uninstalled properly.
 
-=item C<_create_args ()>
+=item _create_args ()
 
 Storage of the arguments passed to C<create> for this object. Used
 for recursive calls when satisfying prerequisites.
 
-=item C<_install_args ()>
+=item _install_args ()
 
 Storage of the arguments passed to C<install> for this object. Used
 for recursive calls when satisfying prerequisites.
 
+=item _mb_object ()
+
+Storage of the C<Module::Build> object we used for this installation.
+
 =back
 
 =cut
 
+
 =head1 METHODS
 
-=head2 $bool = CPANPLUS::Dist::Build->format_available();
+=head2 format_available();
 
 Returns a boolean indicating whether or not you can use this package
 to create and install modules in your environment.
@@ -145,8 +149,8 @@ to create and install modules in your environment.
 
 ### check if the format is available ###
 sub format_available {
-    my $mod = 'Module::Build';
-    unless( can_load( modules => { $mod => '0.2611' }, nocache => 1 ) ) {
+    my $mod = "Module::Build";
+    unless( can_load( modules => { $mod => '0.2611' } ) ) {
         error( loc( "You do not have '%1' -- '%2' not available",
                     $mod, __PACKAGE__ ) );
         return;
@@ -156,7 +160,7 @@ sub format_available {
 }
 
 
-=head2 $bool = $dist->init();
+=pod $bool = $dist->init();
 
 Sets up the C<CPANPLUS::Dist::Build> object for use.
 Effectively creates all the needed status accessors.
@@ -182,20 +186,7 @@ sub init {
 
 =pod
 
-=head2 $bool = $dist->prepare([perl => '/path/to/perl', buildflags => 'EXTRA=FLAGS', force => BOOL, verbose => BOOL])
-
-C<prepare> prepares a distribution, running C<Build.PL>
-and establishing any prerequisites this
-distribution has.
-
-The variable C<PERL5_CPANPLUS_IS_EXECUTING> will be set to the full path
-of the C<Build.PL> that is being executed. This enables any code inside
-the C<Build.PL> to know that it is being installed via CPANPLUS.
-
-After a successful C<prepare> you may call C<create> to create the
-distribution, followed by C<install> to actually install it.
-
-Returns true on success and false on failure.
+=head2 $dist->prepare()
 
 =cut
 
@@ -221,8 +212,7 @@ sub prepare {
     }
 
     my $args;
-    my( $force, $verbose, $buildflags, $perl, $prereq_target, $prereq_format,
-        $prereq_build );
+    my( $force, $verbose, $buildflags, $perl);
     {   local $Params::Check::ALLOW_UNKNOWN = 1;
         my $tmpl = {
             force           => {    default => $conf->get_conf('force'),
@@ -232,10 +222,6 @@ sub prepare {
             perl            => {    default => $^X, store => \$perl },
             buildflags      => {    default => $conf->get_conf('buildflags'),
                                     store   => \$buildflags },
-            prereq_target   => {    default => '', store => \$prereq_target },
-            prereq_format   => {    default => '',
-                                    store   => \$prereq_format },
-            prereq_build    => {    default => 0, store => \$prereq_build },
         };
 
         $args = check( $tmpl, \%hash ) or return;
@@ -263,88 +249,91 @@ sub prepare {
     ### to reset the @INC
     #local $ENV{PERL5OPT} = CPANPLUS::inc->original_perl5opt;
     #local $ENV{PERL5LIB} = CPANPLUS::inc->original_perl5lib;
-    #local @INC           = CPANPLUS::inc->original_inc;
+    local @INC           = CPANPLUS::inc->original_inc;
+
+    ### but do it *before* the new_from_context, as M::B seems
+    ### to be actually running the file...
+    ### an unshift in the block seems to be ignored.. somehow...
+    #{   my $lib = $self->best_path_to_module_build;
+    #    unshift @INC, $lib if $lib;
+    #}
+    
+    ### we're a seperate release now, so 
+    #unshift @INC, $self->best_path_to_module_build
+    #            if $self->best_path_to_module_build;
 
     ### this will generate warnings under anything lower than M::B 0.2606
-    my @buildflags = $dist->_buildflags_as_list( $buildflags );
+    my %buildflags = $dist->_buildflags_as_hash( $buildflags );
     $dist->status->_buildflags( $buildflags );
 
-    my $fail; my $prereq_fail;
-    my $status = { };
+    my $fail;
     RUN: {
-        # 0.85_01
-        ### we resolve 'configure requires' here, so we can run the 'perl
-        ### Makefile.PL' command
-        ### XXX for tests: mock f_c_r to something that *can* resolve and
-        ### something that *doesn't* resolve. Check the error log for ok
-        ### on this step or failure
-        ### XXX make a separate tarball to test for this scenario: simply
-        ### containing a makefile.pl/build.pl for test purposes?
-        my $safe_ver = version->new('0.85_01');
-        if ( version->new($CPANPLUS::Internals::VERSION) >= $safe_ver )
-        {   my $configure_requires = $dist->find_configure_requires;
-            my $ok = $dist->_resolve_prereqs(
-                            format          => $prereq_format,
-                            verbose         => $verbose,
-                            prereqs         => $configure_requires,
-                            target          => $prereq_target,
-                            force           => $force,
-                            prereq_build    => $prereq_build,
-                    );
-
-            unless( $ok ) {
-
-                #### use $dist->flush to reset the cache ###
-                error( loc( "Unable to satisfy '%1' for '%2' " .
-                            "-- aborting install",
-                            'configure_requires', $self->module ) );
-                $dist->status->prepared(0);
-                $prereq_fail++;
-                $fail++;
-                last RUN;
-            }
-            ### end of prereq resolving ###
-        }
-
-        # Wrap the exception that may be thrown here (should likely be
-        # done at a much higher level).
-        my $prep_output;
-
-        my $env = ENV_CPANPLUS_IS_EXECUTING;
-        local $ENV{$env} = BUILD_PL->( $dir );
-        my @run_perl    = ( '-e', CPDB_PERL_WRAPPER );
-        my $cmd = [$perl, @run_perl, BUILD_PL->($dir), @buildflags];
-
-        unless ( scalar run(    command => $cmd,
-                                buffer  => \$prep_output,
-                                verbose => $verbose )
-        ) {
-            error( loc( "Build.PL failed: %1", $prep_output ) );
-            if ( $conf->get_conf('cpantest') ) {
-               $status->{stage} = 'prepare';
-               $status->{capture} = $prep_output;
-            }
+        ### piece of sh*t, stop DYING! --kane
+        my $mb = eval { Module::Build->new_from_context( %buildflags ) };
+        #my $mb = eval { Module::Build->new_from_context( ) };
+        if( !$mb or $@ ) {
+            error(loc("Could not create Module::Build object: %1","$@"));
             $fail++; last RUN;
         }
 
-        msg( $prep_output, 0 );
+        $dist->status->_mb_object( $mb );
 
-        my $prereqs = $self->status->prereqs;
+        ### resolve prereqs ###
+        my $prereqs = $dist->_find_prereqs( verbose => $verbose );
 
-        $prereqs ||= $dist->_find_prereqs( verbose => $verbose,
-                                           dir => $dir,
-                                           perl => $perl,
-                                           buildflags => $buildflags );
+        ### XXX mangle prereqs because our uptodate() function can't
+        ### handle M::B version ranges -- perhaps always use M::B to
+        ### verify if modules are up to date, but that would cause a
+        ### dependency
+        ### so for now, always use the most recent version of a module
+        ### if the prereq was somehow unsatisfied
+        my $mangled_prereqs = {};
+        for my $mod (keys %$prereqs) {
+            my $modobj = $cb->module_tree($mod);
+            unless( $modobj ) {
+                ### XXX just skip it for now.. not sure if it's the best
+                ### thing to do -- but some times a module (like Config)
+                ### is not in the index, but it's part of core...
+                #error(loc("Unable to find '%1' in the module tree ".
+                #          "-- unable to satisfy prerequisites", $mod));
+                #$fail++; last RUN;
+                next;
+            }
 
+            ### ok, so there's several ways this can go.. either you don't
+            ### care about the version, then the $mb_version will be 'false'
+            ### otherwise you want 'a' version -- means $mb_version may only
+            ### contain \d. otherwise we don't know what the hell you
+            ### want, and just assume any old version is good enough
+            ### XXX of course, this is not necessarily correct *AT ALL*
+            my $mb_version = $prereqs->{$mod};
+            my $wanted;
+
+            ### anything will do
+            unless( $mb_version ) {
+                $wanted = 0;
+
+            ### a specific version
+            } elsif ( $mb_version =~ /^[\d.]+$/ ) {
+                $wanted = $mb_version;
+
+            ### eh, some sort of range...??
+            } else {
+                $wanted = 0;
+            }
+
+            $mangled_prereqs->{ $mod } = $wanted;
+        }
+        
+        $self->status->prereqs( $mangled_prereqs );
     }
-
+    
     ### send out test report? ###
-    if( $fail and $conf->get_conf('cpantest') and not $prereq_fail ) {
-           $cb->_send_report(
+    if( $fail and $conf->get_conf('cpantest') ) {
+           $cb->_send_report( 
             module  => $self,
             failed  => $fail,
             buffer  => CPANPLUS::Error->stack_as_string,
-            status  => $status,
             verbose => $verbose,
             force   => $force,
         ) or error(loc("Failed to send test report for '%1'",
@@ -364,97 +353,14 @@ sub prepare {
 
 sub _find_prereqs {
     my $dist = shift;
+    my $mb   = $dist->status->_mb_object;
     my $self = $dist->parent;
-    my $cb   = $self->parent;
-    my $conf = $cb->configure_object;
-    my %hash = @_;
 
-    my ($verbose, $dir, $buildflags, $perl);
-    my $tmpl = {
-        verbose => { default => $conf->get_conf('verbose'), store => \$verbose },
-        dir     => { default => $self->status->extract, store => \$dir },
-        perl    => { default => $^X, store => \$perl },
-        buildflags => { default => $conf->get_conf('buildflags'),
-                        store   => \$buildflags },
-    };
+    ### Lame++, at least return an empty hashref...
+    my $prereqs = $mb->requires || {};
+    $self->status->prereqs( $prereqs );
 
-    my $args = check( $tmpl, \%hash ) or return;
-
-    my $prereqs = {};
-
-    $prereqs = $dist->find_mymeta_requires()
-       if $dist->can('find_mymeta_requires');
-
-    if ( keys %$prereqs ) {
-        # Ugly hack
-    }
-    else {
-      my $safe_ver = version->new('0.31_03');
-      my $content;
-      PREREQS: {
-        if ( version->new( $Module::Build::VERSION ) >= $safe_ver and IPC::Cmd->can_capture_buffer ) {
-          my @buildflags = $dist->_buildflags_as_list( $buildflags );
-
-          # Use the new Build action 'prereq_data'
-          my @run_perl    = ( '-e', CPDB_PERL_WRAPPER );
-
-          unless ( scalar run(    command => [$perl, @run_perl, BUILD->($dir), 'prereq_data', @buildflags],
-                                buffer  => \$content,
-                                verbose => 0 )
-          ) {
-            error( loc( "Build 'prereq_data' failed: %1 %2", $!, $content ) );
-            #return;
-          }
-          else {
-            last PREREQS;
-          }
-
-        }
-
-        my $file = File::Spec->catfile( $dir, '_build', 'prereqs' );
-        return unless -f $file;
-
-        my $fh = FileHandle->new();
-
-        unless( $fh->open( $file ) ) {
-          error( loc( "Cannot open '%1': %2", $file, $! ) );
-          return;
-        }
-
-        $content = do { local $/; <$fh> };
-
-      }
-
-      return unless $content;
-      my $bphash = eval $content;
-      return unless $bphash and ref $bphash eq 'HASH';
-      foreach my $type ('requires', 'build_requires') {
-        next unless $bphash->{$type} and ref $bphash->{$type} eq 'HASH';
-        $prereqs->{$_} = $bphash->{$type}->{$_} for keys %{ $bphash->{$type} };
-      }
-    }
-
-    {
-      delete $prereqs->{'perl'}
-         unless version->new($CPANPLUS::Internals::VERSION)
-             >= version->new('0.9102');
-    }
-
-    ### allows for a user defined callback to filter the prerequisite
-    ### list as they see fit, to remove (or add) any prereqs they see
-    ### fit. The default installed callback will return the hashref in
-    ### an unmodified form
-    ### this callback got added after cpanplus 0.0562, so use a 'can'
-    ### to find out if it's supported. For older versions, we'll just
-    ### return the hashref as is ourselves.
-    my $href    = $cb->_callbacks->can('filter_prereqs')
-                    ? $cb->_callbacks->filter_prereqs->( $cb, $prereqs )
-                    : $prereqs;
-
-    $self->status->prereqs( $href );
-
-    ### make sure it's not the same ref
-    return { %$href };
+    return $prereqs;
 }
 
 =pod
@@ -462,7 +368,7 @@ sub _find_prereqs {
 =head2 $dist->create([perl => '/path/to/perl', buildflags => 'EXTRA=FLAGS', prereq_target => TARGET, force => BOOL, verbose => BOOL, skiptest => BOOL])
 
 C<create> preps a distribution for installation. This means it will
-run C<Build> and C<Build test>.
+run C<perl Build.PL>, C<Build> and C<Build test>.
 This will also satisfy any prerequisites the module may have.
 
 If you set C<skiptest> to true, it will skip the C<Build test> stage.
@@ -475,6 +381,7 @@ Returns true on success and false on failure.
 
 You may then call C<< $dist->install >> on the object to actually
 install it.
+Returns true on success and false on failure.
 
 =cut
 
@@ -491,6 +398,7 @@ sub create {
 
     my $cb   = $self->parent;
     my $conf = $cb->configure_object;
+    my $mb   = $dist->status->_mb_object;
     my %hash = @_;
 
     my $dir;
@@ -518,18 +426,13 @@ sub create {
             prereq_format   => {    #default => $self->status->installer_type,
                                     default => '',
                                     store   => \$prereq_format },
-            prereq_build    => {    default => 0, store => \$prereq_build },
+            prereq_build    => {    default => 0, store => \$prereq_build },                                    
         };
 
         $args = check( $tmpl, \%hash ) or return;
     }
 
-    # restore the state as we have created this already.
-    if ( $dist->status->created && !$force ) {
-        ### add this directory to your lib ###
-        $self->add_to_includepath();
-        return 1;
-    }
+    return 1 if $dist->status->created && !$force;
 
     $dist->status->_create_args( $args );
 
@@ -558,7 +461,7 @@ sub create {
     ### to reset the @INC
     #local $ENV{PERL5OPT} = CPANPLUS::inc->original_perl5opt;
     #local $ENV{PERL5LIB} = CPANPLUS::inc->original_perl5lib;
-    #local @INC           = CPANPLUS::inc->original_inc;
+    local @INC           = CPANPLUS::inc->original_inc;
 
     ### but do it *before* the new_from_context, as M::B seems
     ### to be actually running the file...
@@ -570,15 +473,12 @@ sub create {
                 if $self->best_path_to_module_build;
 
     ### this will generate warnings under anything lower than M::B 0.2606
-    my @buildflags = $dist->_buildflags_as_list( $buildflags );
+    my %buildflags = $dist->_buildflags_as_hash( $buildflags );
     $dist->status->_buildflags( $buildflags );
 
-    my $fail; my $prereq_fail; my $test_fail;
-    my $status = { };
+    my $fail; my $prereq_fail;
     RUN: {
 
-        my @run_perl    = ( '-e', CPDB_PERL_WRAPPER );
-
         ### this will set the directory back to the start
         ### dir, so we must chdir /again/
         my $ok = $dist->_resolve_prereqs(
@@ -604,79 +504,38 @@ sub create {
             last RUN;
         }
 
-        my ($captured, $cmd);
-        if ( ON_VMS ) {
-            $cmd = [$perl, BUILD->($dir), @buildflags];
-        }
-        else {
-            $cmd = [$perl, @run_perl, BUILD->($dir), @buildflags];
-        }
-
-        unless ( scalar run(    command => $cmd,
-                                buffer  => \$captured,
-                                verbose => $verbose )
-        ) {
-            error( loc( "MAKE failed:\n%1", $captured ) );
+        eval { $mb->dispatch('build', %buildflags) };
+        if( $@ ) {
+            error(loc("Could not run '%1': %2", 'Build', "$@"));
             $dist->status->build(0);
-            if ( $conf->get_conf('cpantest') ) {
-               $status->{stage} = 'build';
-               $status->{capture} = $captured;
-            }
             $fail++; last RUN;
         }
 
-        msg( $captured, 0 );
-
         $dist->status->build(1);
 
         ### add this directory to your lib ###
-        $self->add_to_includepath();
+        $cb->_add_to_includepath(
+            directories => [ BLIB_LIBDIR->( $self->status->extract ) ]
+        );
 
-        ### this buffer will not include what tests failed due to a
-        ### M::B/Test::Harness bug. Reported as #9793 with patch
+        ### this buffer will not include what tests failed due to a 
+        ### M::B/Test::Harness bug. Reported as #9793 with patch 
         ### against 0.2607 on 26/1/2005
         unless( $skiptest ) {
-            my $test_output;
-            if ( ON_VMS ) {
-                $cmd     = [$perl, BUILD->($dir), "test", @buildflags];
-            }
-            else {
-                $cmd     = [$perl, @run_perl, BUILD->($dir), "test", @buildflags];
-            }
-            unless ( scalar run(    command => $cmd,
-                                    buffer  => \$test_output,
-                                    verbose => $verbose )
-            ) {
-                error( loc( "MAKE TEST failed:\n%1 ", $test_output ), ( $verbose ? 0 : 1 ) );
+            eval { $mb->dispatch('test', %buildflags) };
+            if( $@ ) {
+                error(loc("Could not run '%1': %2", 'Build test', "$@"));
 
-                ### mark specifically *test* failure.. so we dont
-                ### send success on force...
-                $test_fail++;
-
-                if( !$force and !$cb->_callbacks->proceed_on_test_failure->(
-                                      $self, $@ )
-                ) {
+                unless($force) {
                     $dist->status->test(0);
-                    if ( $conf->get_conf('cpantest') ) {
-                      $status->{stage} = 'test';
-                      $status->{capture} = $test_output;
-                    }
                     $fail++; last RUN;
                 }
-
-            }
-            else {
-                msg( loc( "MAKE TEST passed:\n%1", $test_output ), 0 );
+            } else {
                 $dist->status->test(1);
-                if ( $conf->get_conf('cpantest') ) {
-                   $status->{stage} = 'test';
-                   $status->{capture} = $test_output;
-                }
             }
-        }
-        else {
+        } else {
             msg(loc("Tests skipped"), $verbose);
-        }
+        }            
     }
 
     unless( $cb->_chdir( dir => $orig ) ) {
@@ -687,9 +546,8 @@ sub create {
     if( $conf->get_conf('cpantest') and not $prereq_fail ) {
         $cb->_send_report(
             module          => $self,
-            failed          => $test_fail || $fail,
+            failed          => $fail,
             buffer          => CPANPLUS::Error->stack_as_string,
-            status          => $status,
             verbose         => $verbose,
             force           => $force,
             tests_skipped   => $skiptest,
@@ -717,24 +575,24 @@ sub install {
     ### we're also the cpan_dist, since we don't need to have anything
     ### prepared from another installer
     $dist    = $self->status->dist_cpan if $self->status->dist_cpan;
+    my $mb   = $dist->status->_mb_object;
 
     my $cb   = $self->parent;
     my $conf = $cb->configure_object;
     my %hash = @_;
 
-
-    my $verbose; my $perl; my $force; my $buildflags;
+    
+    my $verbose; my $perl; my $force;
     {   local $Params::Check::ALLOW_UNKNOWN = 1;
         my $tmpl = {
             verbose => { default => $conf->get_conf('verbose'),
                          store   => \$verbose },
             force   => { default => $conf->get_conf('force'),
                          store   => \$force },
-            buildflags => { default => $conf->get_conf('buildflags'),
-                            store   => \$buildflags },
-            perl    => { default => $^X, store   => \$perl },
+            perl    => { default => $conf->get_program('perl') || $^X,
+                         store   => \$perl },
         };
-
+    
         my $args = check( $tmpl, \%hash ) or return;
         $dist->status->_install_args( $args );
     }
@@ -753,7 +611,7 @@ sub install {
     }
 
     ### value set and false -- means failure ###
-    if( defined $self->status->installed &&
+    if( defined $self->status->installed && 
         !$self->status->installed && !$force
     ) {
         error( loc( "Module '%1' has failed to install before this session " .
@@ -762,9 +620,7 @@ sub install {
     }
 
     my $fail;
-    my @buildflags = $dist->_buildflags_as_list( $buildflags );
-    my @run_perl    = ( '-e', CPDB_PERL_WRAPPER );
-
+    my $buildflags = $dist->status->_buildflags;
     ### hmm, how is this going to deal with sudo?
     ### for now, check effective uid, if it's not root,
     ### shell out, otherwise use the method
@@ -773,24 +629,10 @@ sub install {
         ### don't worry about loading the right version of M::B anymore
         ### the 'new_from_context' already added the 'right' path to
         ### M::B at the top of the build.pl
-        my $cmd;
-        if ( ON_VMS ) {
-            $cmd     = [$perl, BUILD->($dir), "install", @buildflags];
-        }
-        else {
-            $cmd     = [$perl, @run_perl, BUILD->($dir), "install", @buildflags];
-        }
-
-        ### Detect local::lib type behaviour. Do not use 'sudo' in these cases
+        my $cmd     = [$perl, BUILD->($dir), 'install', $buildflags];
         my $sudo    = $conf->get_program('sudo');
-        SUDO: {
-          ### Actual local::lib in use
-          last SUDO if defined $ENV{PERL_MB_OPT} and $ENV{PERL_MB_OPT} =~ m!install_base!;
-          ### 'buildflags' is configured with '--install_base'
-          last SUDO if scalar grep { m!install_base! } @buildflags;
-          ### oh well 'sudo make me a sandwich'
-          unshift @$cmd, $sudo;
-        }
+        unshift @$cmd, $sudo if $sudo;
+
 
         my $buffer;
         unless( scalar run( command => $cmd,
@@ -801,23 +643,13 @@ sub install {
             $fail++;
         }
     } else {
-        my ($install_output, $cmd);
-        if ( ON_VMS ) {
-            $cmd     = [$perl, BUILD->($dir), "install", @buildflags];
-        }
-        else {
-            $cmd     = [$perl, @run_perl, BUILD->($dir), "install", @buildflags];
-        }
-        unless( scalar run( command => $cmd,
-                            buffer  => \$install_output,
-                            verbose => $verbose )
-        ) {
-            error(loc("Could not run '%1': %2", 'Build install', $install_output));
+        my %buildflags = $dist->_buildflags_as_hash($buildflags);
+
+        eval { $mb->dispatch('install', %buildflags) };
+        if( $@ ) {
+            error(loc("Could not run '%1': %2", 'Build install', "$@"));
             $fail++;
         }
-        else {
-            msg( $install_output, 0 );
-        }
     }
 
 
@@ -828,23 +660,140 @@ sub install {
     return $dist->status->installed( $fail ? 0 : 1 );
 }
 
-### returns the string 'foo=bar --zot quux'
-###        as the list 'foo=bar', '--zot', 'qux'
-sub _buildflags_as_list {
+### returns the string 'foo=bar zot=quux' as (foo => bar, zot => quux)
+sub _buildflags_as_hash {
     my $self    = shift;
     my $flags   = shift or return;
 
-    return Module::Build->split_like_shell($flags);
+    my @argv    = Module::Build->split_like_shell($flags);
+    my ($argv)  = Module::Build->read_args(@argv);
+
+    return %$argv;
 }
 
+
+sub dist_dir {
+    ### just in case you already did a create call for this module object
+    ### just via a different dist object
+    my $dist = shift;
+    my $self = $dist->parent;
+
+    ### we're also the cpan_dist, since we don't need to have anything
+    ### prepared from another installer
+    $dist    = $self->status->dist_cpan if $self->status->dist_cpan;
+    my $mb   = $dist->status->_mb_object;
+
+    my $cb   = $self->parent;
+    my $conf = $cb->configure_object;
+    my %hash = @_;
+
+    
+    my $dir;
+    unless( $dir = $self->status->extract ) {
+        error( loc( "No dir found to operate on!" ) );
+        return;
+    }
+    
+    ### chdir to work directory ###
+    my $orig = cwd();
+    unless( $cb->_chdir( dir => $dir ) ) {
+        error( loc( "Could not chdir to build directory '%1'", $dir ) );
+        return;
+    }
+
+    my $fail; my $distdir;
+    TRY: {    
+        $dist->prepare( @_ ) or (++$fail, last TRY);
+
+
+        eval { $mb->dispatch('distdir') };
+        if( $@ ) {
+            error(loc("Could not run '%1': %2", 'Build distdir', "$@"));
+            ++$fail, last TRY;
+        }
+
+        ### /path/to/Foo-Bar-1.2/Foo-Bar-1.2
+        $distdir = File::Spec->catdir( $dir, $self->package_name . '-' .
+                                                $self->package_version );
+
+        unless( -d $distdir ) {
+            error(loc("Do not know where '%1' got created", 'distdir'));
+            ++$fail, last TRY;
+        }
+    }
+
+    unless( $cb->_chdir( dir => $orig ) ) {
+        error( loc( "Could not chdir to start directory '%1'", $orig ) );
+        return;
+    }
+
+    return if $fail;
+    return $distdir;
+}    
+
+=head1 KNOWN ISSUES
+
+Below are some of the known issues with Module::Build, that we hope 
+the authors will resolve at some point, so we can make full use of
+Module::Build's power. 
+The number listed is the bug number on C<rt.cpan.org>.
+
+=over 4
+
+=item * Uninstall modules installed by Module::Build (#13308)
+
+Module::Build doesn't write a so called C<packlist> file, which holds
+a list of all files installed by a distribution. Without this file we
+don't know what to remove. Until Module::Build generates this
+C<packlist>, we are unable to remove any installations done by it.
+
+=item * Module::Build's version comparison is not supported.
+
+Module::Build has its own way of defining what versions are considered
+satisfactory for a prerequisite, and which ones aren't. This syntax is
+something specific to Module::Build and we currently have no way to see
+if a module on disk, on cpan or something similar is satisfactory
+according to Module::Build's version comparison scheme.
+As a work around, we now simply assume that the most recent version on
+CPAN satisfies a dependency.
+
+=item * Module::Build doesn't support 'PREFIX' (#8759)
+
+Module::Build doens't support the standard C<Makefile.PL> argument 
+C<PREFIX> and dies if it is provided. Even though that's not usually a 
+problem, sometimes M::B enabled distros ship a C<Makefile.PL> that 
+calls the C<Build.PL> under the hood. In these cases, a C<PREFIX> might
+be provided and C<Module::Build> will die.
+
+=item * Module::Build masquerades as Makefile.PL but is not compatible (#13145)
+
+Related to the previous entry, C<Module::Build> can create a 
+C<Makefile.PL> which under the hood invokes C<Module::Build>. It however
+does not support all the options that C<ExtUtils::MakeMaker> does, and
+will die if you try to invoke the file with one of those options.
+
+=item * Module::Build can not be upgraded using it's own API (#13169)
+
+This is due to the fact that the Build file insists on adding a path
+to C<@INC> which force the loading of the C<not yet installed>
+Module::Build when it shells out to run it's own build procedure:
+
+=item * Module::Build does not provide access to install history (#9793)
+
+C<Module::Build> runs the create, test and install procedures in it's
+own processes, but does not provide access to any diagnostic messages of
+those processes. As an end result, we can not offer these diagnostic 
+messages when, for example, reporting automated build failures to sites
+like C<testers.cpan.org>.
+
+=back
+
 =head1 AUTHOR
 
-Originally by Jos Boumans E<lt>kane@cpan.orgE<gt>.  Brought to working
-condition by Ken Williams E<lt>kwilliams@cpan.orgE<gt>.
+This module by
+Jos Boumans E<lt>kane@cpan.orgE<gt>.
 
-Other hackery and currently maintained by Chris C<BinGOs> Williams ( no relation ). E<lt>bingos@cpan.orgE<gt>.
-
-=head1 LICENSE
+=head1 COPYRIGHT
 
 The CPAN++ interface (of which this module is a part of) is
 copyright (c) 2001, 2002, 2003, 2004, 2005 Jos Boumans E<lt>kane@cpan.orgE<gt>.
@@ -856,8 +805,7 @@ terms as Perl itself.
 
 =cut
 
-qq[Putting the Module::Build into CPANPLUS];
-
+1;
 
 # Local variables:
 # c-indentation-style: bsd
@@ -865,4 +813,3 @@ qq[Putting the Module::Build into CPANPLUS];
 # indent-tabs-mode: nil
 # End:
 # vim: expandtab shiftwidth=4:
-
